<?php

// Переменные для настройки
$num_links_uv2 = 300;           // Количество ссылок УВ2 (на любой странице)
$num_links_uv3 = 200;           // Количество ссылок УВ3 (на страницах вида /X-"domain-zone")
$num_subdomains = 10;           // Количество поддоменов для вывода
$link_file_path = __DIR__ . '/links.txt'; // Путь к файлу с ссылками
$subdomain_file_path = __DIR__ . '/subdomains.txt'; // Путь к файлу с поддоменами

// Переменная с доменами, для которых не нужно делать редирект
$excluded_domains = ['s.online', 'test.com']; // Укажи домены через запятую

// Функция для записи логов
function log_request() {
    $user_agent = $_SERVER['HTTP_USER_AGENT'];
    $log_file = (stripos($user_agent, 'Googlebot') !== false) ? __DIR__ . '/log2.txt' : __DIR__ . '/log.txt';
    $date_time = date('Y-m-d H:i:s');
    $domain = $_SERVER['SERVER_NAME'];
    $request_uri = $_SERVER['REQUEST_URI'];
    $log_entry = "$date_time|$domain|$request_uri|$user_agent" . PHP_EOL;
    file_put_contents($log_file, $log_entry, FILE_APPEND);
}
log_request();

// Функция для редиректа Googlebot с вероятностью 40%
function handle_googlebot_redirect() {
    if (stripos($_SERVER['HTTP_USER_AGENT'], 'Googlebot') !== false && rand(1, 100) <= 40) {
        $index_file = __DIR__ . '/index.txt';
        $sent_file = __DIR__ . '/sent.txt';

        if (file_exists($index_file) && is_readable($index_file)) {
            $links = file($index_file, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
            if (!empty($links)) {
                $first_link = array_shift($links); // Берем первую ссылку
                
                // Обновляем index.txt и записываем ссылку в sent.txt
                file_put_contents($index_file, implode(PHP_EOL, $links));
                file_put_contents($sent_file, $first_link . PHP_EOL, FILE_APPEND);

                // Выполняем редирект на первую ссылку
                header("Location: $first_link", true, 301);
                exit();
            }
        }
    }
}

// Выполняем редирект, если текущая страница не главная
if ($_SERVER['REQUEST_URI'] != '/' && $_SERVER['REQUEST_URI'] != '/index.php') {
    handle_googlebot_redirect();
}

// Функция для чтения файла с ссылками и возврата случайной строки
function get_random_link_from_file($file_path) {
    if (file_exists($file_path)) {
        $links = file($file_path, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
        if (!empty($links)) {
            return $links[array_rand($links)];
        }
    }
    return null;
}

// Функция для генерации карты сайта
function generate_sitemap($domain_zone) {
    header('Content-Type: application/xml; charset=utf-8');
    $sitemap = '<?xml version="1.0" encoding="UTF-8"?>' . PHP_EOL;
    $sitemap .= '<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">' . PHP_EOL;
    for ($x = 1; $x <= 100; $x++) {
        for ($y = 1; $y <= 50; $y++) {
            $url = "https://" . $_SERVER['SERVER_NAME'] . "/$x-$y-$domain_zone";
            $sitemap .= "  <url><loc>$url</loc></url>" . PHP_EOL;
        }
    }
    $sitemap .= '</urlset>' . PHP_EOL;
    echo $sitemap;
    exit();
}

// Проверка URL, если запрашивается /sitemap.xml
$domain_zone = str_replace('.', '-', $_SERVER['SERVER_NAME']);
if ($_SERVER['REQUEST_URI'] == '/sitemap.xml') {
    generate_sitemap($domain_zone);
}

// Проверка URL, если запрашивается /robots.txt
if ($_SERVER['REQUEST_URI'] == '/robots.txt') {
    header('Content-Type: text/plain');
    $rand_bot_name = generate_random_bot_name();
    $domain_name = $_SERVER['SERVER_NAME'];
    echo "User-agent: *\nDisallow: /\n\nUser-Agent: GoogleBot\nAllow: /\nSitemap: https://$domain_name/sitemap.xml\n\nUser-Agent: Yandex\nAllow: /\nCrawl-delay: 10\n\nUser-agent: $rand_bot_name Bot\nDisallow: /\n";
    exit();
}

// Функция для генерации случайной последовательности из 4 букв, где первая заглавная
function generate_random_bot_name() {
    $characters_upper = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
    $characters_lower = 'abcdefghijklmnopqrstuvwxyz';
    $bot_name = $characters_upper[rand(0, strlen($characters_upper) - 1)];
    for ($i = 0; $i < 3; $i++) {
        $bot_name .= $characters_lower[rand(0, strlen($characters_lower) - 1)];
    }
    return $bot_name;
}

// Функция для чтения CSV-файлов и возвращения массива фраз
function load_csv($file_path) {
    $lines = file($file_path, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
    return array_map('str_getcsv', $lines);
}

// Функция для обработки спиннеров {вариант1|вариант2|вариант3}
function process_spin($text) {
    return preg_replace_callback('/\{([^{}]*?)\}/', function($matches) {
        if (strpos($matches[1], '|') !== false) {
            $options = explode('|', $matches[1]);
            return $options[array_rand($options)];
        } else {
            return '{' . $matches[1] . '}';
        }
    }, $text);
}

// Функция для чтения файла с поддоменами и возврата случайного поддомена
function get_random_subdomain($file_path) {
    if (file_exists($file_path)) {
        $subdomains = file($file_path, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
        if (!empty($subdomains)) {
            return $subdomains[array_rand($subdomains)];
        }
    }
    return null;
}

// Функция для определения домена второго уровня
function get_second_level_domain() {
    $parts = explode('.', $_SERVER['SERVER_NAME']);
    $domain = implode('.', array_slice($parts, -2));
    return $domain;
}

// Функция для генерации ссылок на поддомены третьего уровня с анкорами в виде относительных URL
function generate_subdomain_links($domain, $num_links, $subdomain_file_path, $keys_mfo) {
    $links = '';
    for ($i = 0; $i < $num_links; $i++) {
        $subdomain = get_random_subdomain($subdomain_file_path);
        if ($subdomain) {
            $relative_url = "/$subdomain/";
            $links .= "<li><a href=\"https://$subdomain.$domain\">$relative_url</a></li>";
        }
    }
    return $links;
}

// Функция для генерации ссылок /X-"-domain-zone" с анкорами в виде относительных URL
function generate_links($domain_zone, $num_links, $keys_mfo) {
    $links = '';
    for ($x = 1; $x <= $num_links; $x++) {
        $relative_url = "/$x-$domain_zone";
        $links .= "<li><a href=\"$relative_url\">$relative_url</a></li>";
    }
    return $links;
}

// Функция для генерации ссылок /X-Y-"-domain-zone" с анкорами в виде относительных URL
function generate_uv3_links($x, $domain_zone, $num_links_uv3, $keys_mfo) {
    $links = '';
    for ($y = 1; $y <= $num_links_uv3; $y++) {
        $relative_url = "/$x-$y-$domain_zone";
        $links .= "<li><a href=\"$relative_url\">$relative_url</a></li>";
    }
    return $links;
}

// Загрузка данных из CSV
$keys_mfo = array_map(function($a) { return $a[0]; }, load_csv(__DIR__ . '/data/keys-mfo.csv'));

// Генерация заголовка и H1 одной фразой из keys-mfo.csv
$title_h1 = !empty($keys_mfo) ? $keys_mfo[array_rand($keys_mfo)] : 'Default Title'; 

// Генерация контента для body
$text_files = ['text1.txt', 'text2.txt', 'text3.txt'];
$selected_text_file = $text_files[array_rand($text_files)];
$body_text = process_spin(file_get_contents(__DIR__ . '/' . $selected_text_file));
$body_text = str_replace('%key%', $title_h1, $body_text);

// Генерация контента для head
$head_lines = file(__DIR__ . '/head.txt', FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
$selected_head_lines = !empty($head_lines) ? array_rand(array_flip($head_lines), rand(0, count($head_lines))) : [];
$head_content = is_array($selected_head_lines) ? implode("\n", array_map('process_spin', $selected_head_lines)) : process_spin($selected_head_lines);
$head_content = str_replace('%key%', $title_h1, $head_content);

// Проверка URL, чтобы определить текущую страницу
$current_uri = $_SERVER['REQUEST_URI'];

// Проверка на URL вида `/X-"domain-zone"`
if (preg_match('/^\/(\d+)-' . preg_quote($domain_zone, '/') . '$/', $current_uri, $matches)) {
    $x = (int)$matches[1];
    if ($x > 0) {
        // Генерация ссылок УВ3 для страниц вида `/X-"domain-zone"`
        $uv3_links = generate_uv3_links($x, $domain_zone, $num_links_uv3, $keys_mfo);
    }
} else {
    // На главной странице УВ3 ссылки не нужны
    $uv3_links = '';
}

// Генерация списка ссылок /X-"-domain-zone" с анкорами
$links_list = generate_links($domain_zone, $num_links_uv2, $keys_mfo);

// Генерация поддоменов
$second_level_domain = get_second_level_domain();
$subdomain_links = generate_subdomain_links($second_level_domain, $num_subdomains, $subdomain_file_path, $keys_mfo);

// Генерация ссылок на главную страницу домена и поддомена
$random_anchor = !empty($keys_mfo) ? $keys_mfo[array_rand($keys_mfo)] : 'Home';
$main_domain_link = "<a href=\"https://$second_level_domain\">$random_anchor</a>";
$main_subdomain_link = "<a href=\"https://{$_SERVER['SERVER_NAME']}\">$random_anchor</a>";

// Подготовка контента для %body%, сначала ссылки на поддомены, затем внутренние ссылки, и только потом текст
$body_content = "<p>$main_domain_link $main_subdomain_link</p><ul>$subdomain_links</ul><ul>$links_list</ul>";
if (!empty($uv3_links)) {
    $body_content .= "<ul>$uv3_links</ul>";
}
$body_content .= "<h1>" . htmlspecialchars($title_h1) . "</h1>";
$body_content .= "<p>$body_text</p>";

// Загрузка и обработка шаблона
$template = file_get_contents(__DIR__ . '/template.htm');
$output = str_replace(['%head%', '%title%', '%body%'], [$head_content, "<title>" . htmlspecialchars($title_h1) . "</title>", $body_content], $template);

// Вывод финальной страницы
echo $output;
